/**
 * Faction.java -  contains MMS Faction data for a single faction
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
 * Contains MMS Faction data for a single faction
 * @author  Michael Latham
 */
public class Faction implements Cloneable {

    // Constants
    final int M_NO              = 0;
    final int M_YES             = 1;
    final int M_CANCEL          = 2;

    // Attributes
    private String          m_factionName;
    private Vector<String>  m_properties;
    
    /** Default constructor - creates an empty Faction; not preferred */
    public Faction () {
        m_factionName   = "";
        m_properties    = new Vector<String>();
    }
    
    /** Preferred constructor - creates faction with the given name */
    public Faction (String factionName) {
        m_factionName   = factionName;
        m_properties    = new Vector<String>();
    }
    
    /**
     * Clone class for a deep copy
     * @return Copy of this Faction object
     */
    public Faction clone() {
        String newName = new String(m_factionName);
        Faction newFaction = new Faction( newName );
        Vector<String> newPropVector = new Vector<String>();

        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            String newProp = new String(fParam);
            newFaction.addFactionProperty( newProp );
        }
        
        return newFaction;
    }
    
    /** @return name of Faction */
    public String getName() {
        return m_factionName;
    }
    
    /**
     * Given the string for the property, add it to the list of properties
     * @param mmsLine   The line for the property (e.g. Unit_Rangers_1=Stone Rhino SERO,Capt Black,3,3)
     */
    public void addFactionProperty( String mmsLine ) {
        m_properties.addElement(mmsLine);
    }
    
    /**
     * Return the element at the given index (zero based) as a String
     * @param idx   Zero based index
     * @return Element as a String at index, or null
     */
    public String getFactionProperty( int idx ) {
        if (idx < 0 || idx > m_properties.size())
            return null;
        String str = m_properties.get(idx);
        return str;
    }
    
    /**
     * Adds all the Faction properties to the given vector
     * @param vList     String vector that will receive all Faction properties
     */
    public void addAllProperties( Vector<String> vList ) {
        if (vList == null)
            return;
        if (m_properties.size() > 0)
            vList.addAll( m_properties );
    }
    
    /**
     * Add all units to the given vector
     * @param vList     String vector that will receive list of Units
     */
    public void addUnits( Vector<String> vList ) {
        if (vList == null)
            return;

        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (propertyIsUnit( fParam )) {
                fUnit = getUnitValue( fParam );
                vList.add( fUnit );
            }
        }
    }
    
    /**
     * Add all non-unit declarations to the given vector
     * @param vList     String vector that will receive list of Units
     * @param unitIdx   Unit number; a number less than 1 implies get all unit properties
     */
    public void addUnitProperties( Vector<String> vList, int unitIdx ) {
        if (vList == null)
            return;

        String fUnit;
        String fParam;
        int unitIdxMatch;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (propertyIsUnit( fParam ) == false && fParam.indexOf("Location") < 0) {
                unitIdxMatch = fParam.indexOf( "Unit_" + m_factionName + "_" + unitIdx );
                if (unitIdxMatch >= 0 || unitIdx < 1) {
                    fUnit = getPropertyValue( fParam );
                    vList.add( fUnit );
                }
            }
        }
    }

    /**
     * Determine if given property is a Unit declaration
     * @param lParam    Property to check
     * @return true if Unit declaration; false otherwise
     */
    public boolean propertyIsUnit( String lParam ) {
        boolean isUnit = false;
        int idx = lParam.indexOf("=");
        if (idx > 0) {
            String lUnitDec = "Unit_" + m_factionName + "_";
            int lDecLength = lUnitDec.length();
            String lDec = lParam.substring(0, idx);
            lDec = lDec.trim();
            if (lDecLength < lDec.length()) {
                lDec = lDec.substring(lDecLength);
                //if what's left is a number, then we have a Unit declaration
                try {
                    Integer unitNum = new Integer(lDec);
                    isUnit = true;
                } catch( Exception e ) {
                    isUnit = false;
                }
            }
        }
        return isUnit;
    }

    /**
     * Return the Unit value for the given property string
     * @param lParam    Full property string
     * @return Value of the property, or blanks if invalid format
     */
    public String getUnitValue( String lParam ) {
        String lValue = "";
        int idx = lParam.indexOf("=");
        if (idx >= 0 && idx < lParam.length()) {
            lValue = lParam.substring( idx+1 );
        }
        return lValue;
    }

    /**
     * Return the propery value for the given property string
     * @param lParam    Full property string
     * @return String   Value of the property, or blanks if invalid format
     */
    public String getPropertyValue( String lParam ) {
        String lValue = "";
        String tmpStr = "";
        int idx = lParam.indexOf("=");
        if (idx >= 0) {
            for (int j = idx-1; j >= 0; j--) {
                tmpStr = lParam.substring( j, j+1 );
                if (tmpStr.compareTo("_") == 0) {
                    lValue = lParam.substring( j+1 );
                    break;
                }
            }
        }
        return lValue;
    }

    /**
     * @return Value of the location, or blanks if not set
     */
    public String getLocation() {
        String lValue = "";
        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (fParam.indexOf("Location") >= 0) {
                lValue = getUnitValue( fParam );
                break;
            }
        }
        return lValue;
    }
    
    /**
     * Remove the given property from the list
     * @param prop  Entire property string
     * @return true if removed; false if error or not found
     */
    public boolean removeProperty( String prop ) {
        boolean lOk = false;
        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (fParam.compareTo( prop ) == 0) {
                m_properties.remove(j);
                lOk = true;
                break;
            }
        }
        return lOk;
    }
    
    /**
     * Add a new Unit to the list
     * @param newUnit   Unit value of format: mech,pilot,x,y,...
     */
    public void addUnit( String newUnit ) {
        int ct = getUnitCount() + 1;
        String newProp = "Unit_" + m_factionName + "_" + ct + "=" + newUnit;
        m_properties.add( newProp );
    }
    
    /**
     * Replace a Unit with new Unit info
     * @param oldUnit - current Unit to replace
     * @param newUnit - new Unit description
     */
    public void replaceUnit( String oldUnit, String newUnit ) {
        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        boolean searching = true;
        for (int j = 0; j < vSize && searching; j++) {
            fParam = m_properties.get(j);
            if (propertyIsUnit( fParam )) {
                fUnit = getUnitValue( fParam );
                if (fUnit.compareTo(oldUnit) == 0) {
                    //Found it, j is the index
                    int idx = fParam.indexOf("=");
                    String unitStr = fParam.substring(0,idx+1) + newUnit;
                    //m_properties.setElementAt(unitStr, j);
                    m_properties.removeElementAt(j);
                    m_properties.add( j, unitStr );
                    searching = false;
                }
            }
        }
    }
    
    /**
     * @return Total number of units declared
     */
    public int getUnitCount() {
        Vector<String> factionUnits = new Vector<String>();
        addUnits( factionUnits );
        int ct = factionUnits.size();
        return ct;
    }
    
    /**
     * Remove the Unit and all its properties
     * @param oldUnitNum    Unit number
     */
    public void removeUnit( int oldUnitNum ) {
        String fParam;
        String oldUnit1 = "Unit_" + m_factionName + "_" + oldUnitNum + "=";
        String oldUnit2 = "Unit_" + m_factionName + "_" + oldUnitNum + "_";
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (fParam.indexOf( oldUnit1 ) >= 0 || fParam.indexOf( oldUnit2 ) >= 0) {
                m_properties.removeElementAt( j );
                //Back up, since we just altered the list and set our new size
                j--;
                vSize--;
            }
        }
    }
    
    /**
     * Adds a new property for the given Unit
     * @param unitIdx   Unit number
     * @param propVal   Property and value
     */
    public void addUnitProperty( int unitIdx, String propVal ) {
        if (unitIdx < 1 || propVal.length() < 0)
            return;
        String newProp = "Unit_" + m_factionName + "_" + unitIdx + "_" + propVal;
        m_properties.add( newProp );
    }

    /**
     * Replace property for the given Unit and property
     * @param unitIdx   Unit number
     * @param oldVal    Old property and value
     * @param newVal    New property and value
     */
    public void replaceUnitProperty( int unitIdx, String oldVal, String newVal ) {
        if (unitIdx < 1 || newVal.length() < 0)
            return;
        String oldProp = "Unit_" + m_factionName + "_" + unitIdx + "_" + oldVal;
        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        boolean searching = true;
        for (int j = 0; j < vSize && searching; j++) {
            fParam = m_properties.get(j);
            if (fParam.compareTo(oldProp) == 0) {
                //Found it, j is the index
                String newProp = "Unit_" + m_factionName + "_" + unitIdx + "_" + newVal;
                m_properties.removeElementAt(j);
                m_properties.add( j, newProp );
                searching = false;
            }
        }
    }
     
    /**
     * Sets the location property to the given location
     * @param newLoc    Location (N, NE, S, etc.)
     */
    public void setLocation( String newLoc ) {
        if (newLoc.compareTo( getLocation() ) == 0)
            return;
        String fUnit;
        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            if (fParam.indexOf("Location") >= 0) {
                m_properties.removeElementAt(j);
                break;
            }
        }
        String newLocProp = "Location_" + m_factionName + "=" + newLoc;
        m_properties.add( newLocProp );
    }
    
    /**
     * Replaces all properties with the new name
     * @param newName   New name of Faction
     */
    public void changeName( String newName ) {
        if (newName.length() < 0 || newName.compareTo(m_factionName) == 0)
            return;

        String fUnit;
        String fParam;
        String fVal;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            fParam = fParam.replace( m_factionName, newName );
            m_properties.removeElementAt(j);
            m_properties.add( j, fParam );
        }
        m_factionName = newName;
    }
    
    /**
     * Return this faction for the output file buffer of a scenario
     * @return Scenario file buffer for this Faction
     */
    public String getFactionForScenarioFile() {
        String fBuf = "";
        String lineSep = System.getProperty("line.separator");
        String fParam;
        int vSize = m_properties.size();
        for (int j = 0; j < vSize; j++) {
            fParam = m_properties.get(j);
            fBuf += fParam + lineSep;
        }
        
        return fBuf;
    }
}
