/*
 * MmsEditValueDialog.java -  Dialog class to edit a single value
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicOptionPaneUI.*;

/**
 * Dialog class to edit a single value.  Three buttons, OK, Cancel, and Help are added initially, and more may
 * be added by the caller.  The help message may be provided with the constructor, and will be shown automatically
 * when the Help button is clicked.
 * @author  Michael Latham
 */
public class MmsEditValueDialog extends JDialog implements MouseListener{

    /**
     * Constants
     */
    final int M_NO      = 0;
    final int M_YES     = 1;
    final int M_CANCEL  = 2;
    final int M_HELP    = 3;

    Container   m_cp        = null;
    JPanel      m_pData     = null;
    JPanel      m_pButtons  = null;
    JTextField  m_tfValue   = null;
    String      m_helpMsg   = "";
    String      m_value;
    int         m_answer;
    int         m_btnCnt;
    
    /**
     * Default constructor
     */
    public MmsEditValueDialog () {
        super();
        initialize();
    }
     
    /**
     * Preferred constructor
     * @param frame JFrame object of parent
     * @param title Title of new dialog
     * @param helpMsg   Message for help button dialog, if any
     * @param modal True for new dialog to be modal
     */
    public MmsEditValueDialog ( JFrame frame, String title, String helpMsg, boolean modal ) {
        super( frame, title, modal );
        initialize();
        setHelpMessage(helpMsg);
    }

    /**
     * Alternate preferred constructor
     * @param dlg       Dialog object of parent
     * @param title     Title of new dialog
     * @param helpMsg   Message for help button dialog, if any
     */
    public MmsEditValueDialog ( JDialog dlg, String title, String helpMsg ) {
        super( dlg, title, true );
        initialize();
        setHelpMessage(helpMsg);
    }
     
     
    /**
     * Called after constructing to prepare for dialog
     */
    private void initialize() {
        m_cp = getContentPane();
        m_cp.setLayout( new BorderLayout(10,10) );
        m_pButtons  = new JPanel(new ButtonAreaLayout(true, 10));
        m_pData     = new JPanel();
        m_tfValue   = new JTextField();
        m_tfValue.setColumns(28);
        m_value     = "";

        Dimension paneSize = new Dimension( 350, 110 );
        setMinimumSize( paneSize );
        setPreferredSize( paneSize );
        setSize( paneSize );
        Window win = getOwner();
        if (win != null) {
            Point loc = win.getLocation();
            setLocation( loc.x + 10, loc.y + 10 );
        }
        else {
            setLocation( 200, 100 );
        }

        m_answer = this.M_CANCEL;
        m_btnCnt = 0;
        addButton("Ok");
        addButton("Cancel");
        addButton("Help");
        m_pData.add( m_tfValue );

        m_cp.add( m_pData, BorderLayout.NORTH );
        m_cp.add( m_pButtons, BorderLayout.CENTER );
    }
    
    /**
     * Adds a new button and its corresponding answer to the dialog
     * @param newButtonText Text to be shown on new button.  The button's number index will be returned in m_answer
     */
    public int addButton( String newButtonText ) {
        if (newButtonText == null || newButtonText.length() < 1)
            return -1;

        m_btnCnt++;
        String btnName = Integer.toString(m_btnCnt);
        JButton newButton = new JButton( newButtonText );
        newButton.setName( btnName );
        newButton.addMouseListener(this);
        m_pButtons.add( newButton );
        repaint();
        
        return m_btnCnt;
    }
    
    /**
     * Adds or sets the data to the dialog
     * @param data  Text to display in text field, if any
     */
    public void addData( String data ) {
        if (data != null && data.length() > 0) {
            m_value = data;
            m_tfValue.setText(m_value);
        }
        
        repaint();
    }
    
    /**
     * Get the raw data value from the dialog
     * @return Text data value being displayed
     */
    public String getData() {
        String data = m_tfValue.getText();
        return data;
    }
    
    /**
     * Returns the answer as set by last button clicked
     * @return Button index, where user buttons are > 2
     */
    public int getAnswer() {
        return m_answer;
    }
    
    /**
     * Returns the text in the text field
     * @return Value set in edit field
     */
    public String getValue() {
        return m_value;
    }
    
    /**
     * Sets the help message string, which will popup if the Help button is clicked
     * @param lHelp Message to be displayed when help button is clicked
     */
    public void setHelpMessage( String lHelp ) {
        m_helpMsg = lHelp;
    }
    
    /**
     * Pop up simple dialog to show a Help message
     */
    private void showHelp() {
        if (m_helpMsg.compareTo("") == 0) {
            m_helpMsg = "No help available.";
        }
        MmsQuestion ques = new MmsQuestion( this, "Help Dialog" );
        ques.addButton( "Close" );
        ques.addQuestion( m_helpMsg );
        ques.setVisible( true );
        ques.dispose();
    }
     
    /** 
     * Mouse click event handler
     * @param me    MouseEvent from a mouse button click
     */
    public void mouseClicked( MouseEvent me ) {
        JButton bClicked = (JButton) me.getSource();
        
        if (bClicked == null)
            return;
        
        String strAnswer = bClicked.getName();
        
        if (bClicked.getText().toLowerCase().compareTo("help") == 0) {
            showHelp();
            return;
        }
        
        m_answer = Integer.parseInt(strAnswer);
        m_value = m_tfValue.getText();

        setVisible(false);
    }

    /** mouseEntered - N/A -    Invoked when the mouse enters a component. */
    public void mouseEntered( MouseEvent me ) {
    }

    /** mouseEntered - N/A -    Invoked when the mouse exits a component. */
    public void mouseExited( MouseEvent me )  {
    }
  
    /** mousePressed - N/A - Invoked when a mouse button has been pressed on a component. */
    public void mousePressed( MouseEvent me ) {
    }
  
    /** mouseReleased - N/A - Invoked when a mouse button has been released on a component. */
    public void mouseReleased( MouseEvent me ) {
    }
}
