/*
 * MmsEditor.java -  Editor GUI tool for MegaMek MMS files to create and edit scenarios
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicOptionPaneUI.*;

/**
 * Editor GUI tool for MegaMek MMS files to create and edit scenarios.
 * @author  Michael Latham
 */
public class MmsEditor extends JFrame implements MouseListener, ItemListener {

    /**
     * Constants
     */
    final int M_NO      = 0;
    final int M_YES     = 1;
    final int M_CANCEL  = 2;

    /**
     * Attributes
     */
    //Contentent container for this JFrame
    Container m_cp;
    //default horizontal spacing for RigidArea
    Dimension m_defHorSpacing;
    //default vertical spacing for RigidArea
    Dimension m_defVerSpacing;
    //true = edits in process
    boolean   m_editsStarted;
    //true = output debug messages
    boolean   m_debug;
    Scenario  m_scenario;
    
    //North components
    JComboBox   m_cbVersion;
    JTextField  m_tfName;
    JTextField  m_tfDesc;
    JTextField  m_tfWidth;
    JTextField  m_tfHeight;
    JTextField  m_tfMapWidth;
    JTextField  m_tfMapHeight;
    JTextField  m_tfMaps;
    JComboBox   m_cbFactions;
    
    //Center components
    JList m_listFactions;
    
    //South components
    JButton m_bLoadScenario;
    JButton m_bAddScenario;
    JButton m_bAddFaction;
    JButton m_bEditFaction;
    JButton m_bDeleteFaction;
    JButton m_bSave;
    JButton m_bClose;
    JTextArea m_taDebugOutput;


    /**
     * Constructor - instantiates the class
     */
    public MmsEditor () {
        //Initialize JFrame
        frameInit();
        m_cp = null;
        m_defHorSpacing = new Dimension( 10,0 );
        m_defVerSpacing = new Dimension( 0,10 );
        m_editsStarted  = false;
        m_debug         = false;
        m_scenario      = null;
    }
    
    /**
     * Launches MmsEditor GUI
     * @param args - String array of arguments passed from MmsStart.main
     */
    public void start( String[] args ) {
        createEditorGUI();        
    }
    
    /**
     * Builds and shows MmsEditor GUI frame and components 
     */
    private void createEditorGUI() {
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                quit();
            }
        });

        setTitle("MmsEditor");
        Dimension paneSize = new Dimension( 500, 500 );
        setMinimumSize( paneSize );
        setPreferredSize( paneSize );
        setSize( paneSize );
        setLocation( 200, 100 );

        m_cp    = getContentPane();
        m_cp.setLayout( new BorderLayout(10,10) );
        
        addPanelNorthDataEntry();
        addPanelCenterFactionList();
        addPanelSouthButtons();
        
        //Buffer East and West panels
        m_cp.add( Box.createRigidArea(m_defHorSpacing), BorderLayout.EAST );
        m_cp.add( Box.createRigidArea(m_defHorSpacing), BorderLayout.WEST );
        
        setVisible(true);
    }
     
    /** 
     * Mouse click event handler
     * @param me    Mouse Event generated from mouse button click
     */
    public void mouseClicked( MouseEvent me ) {
        JButton bClicked = (JButton) me.getSource();
        
        if (bClicked == null)
            return;
        
        //Since MouseEvents are not prevented for disabled components
        if (bClicked.isEnabled() == false)
            return;
        
        String bId = bClicked.getText();
        
        if (bId == "Load Scenario" ) {
            if (m_editsStarted) {
                int doSave = askToSave();
                if (doSave == M_YES) {
                    doSave = saveMms();
                }
                if (doSave == M_CANCEL) {
                    //Do nothing and bail
                    return;
                }
            }
            if (loadMms() == M_YES)
                displayMms();
            else
                return;
            m_bLoadScenario.setEnabled(true);
            m_bAddScenario.setEnabled(true);
            m_bAddFaction.setEnabled(true);
            m_bEditFaction.setEnabled(true);
            m_bDeleteFaction.setEnabled(true);
            m_bSave.setEnabled(true);
            m_bClose.setEnabled(true);
        }
        else if (bId == "Add Scenario" ) {
            if (m_editsStarted) {
                int doSave = askToSave();
                if (doSave == M_YES) {
                    doSave = saveMms();
                }
                if (doSave == M_CANCEL) {
                    //Do nothing and bail
                    return;
                }
            }
            m_editsStarted  = true;
            m_bLoadScenario.setEnabled(true);
            m_bAddScenario.setEnabled(true);
            m_bAddFaction.setEnabled(true);
            m_bEditFaction.setEnabled(false);
            m_bDeleteFaction.setEnabled(false);
            m_bSave.setEnabled(true);
            m_bClose.setEnabled(true);
            if (m_tfName.getText().length() > 0) {
                if (askToKeep() == false)
                    m_scenario = new Scenario( "New Scenario", true );
                else
                    m_scenario.setAsNew();
            }
            else
                m_scenario = new Scenario( "New Scenario", true );

            displayMms();
        }
        else if (bId == "Add Faction" ) {
            m_editsStarted  = true;
            Faction lFaction = new Faction("New Faction");
            MmsFactionDialog factionDlg = new MmsFactionDialog( this, lFaction, true );
            factionDlg.setVisible(true);
            if (factionDlg.getAnswer() == factionDlg.M_YES) {
                Faction newFaction = factionDlg.getFaction();
                m_scenario.addFaction( newFaction );
                m_cbFactions.addItem( newFaction.getName() );
                String sel = (String)m_cbFactions.getSelectedItem();
                m_scenario.buildFactionListDisplay( m_listFactions, sel );
                m_bEditFaction.setEnabled(true);
                m_bDeleteFaction.setEnabled(true);
            }
            factionDlg.dispose();
        }
        else if (bId == "Edit Faction" ) {
            Faction lFaction = getSelectedFaction();
            if (lFaction != null) {
                m_editsStarted  = true;
                MmsFactionDialog factionDlg = new MmsFactionDialog( this, lFaction, true );
                factionDlg.setVisible(true);
                
                if (factionDlg.getAnswer() == factionDlg.M_YES) {
                    Faction newFaction = factionDlg.getFaction();
                    m_scenario.replaceFaction( lFaction, newFaction );
                    m_scenario.buildFactionNameComboBox( m_cbFactions, true, true );
                    String sel = (String)m_cbFactions.getSelectedItem();
                    m_scenario.buildFactionListDisplay( m_listFactions, sel );
                }

                factionDlg.dispose();
            }
        }
        else if (bId == "Delete Faction" ) {
            Faction lFaction = getSelectedFaction();
            if (lFaction == null)
                return;

            MmsQuestion ask = new MmsQuestion(this, "Delete Faction Dialog", true);
            ask.initialize();
            ask.addQuestion("Delete selected faction?");
            ask.addButton( new JButton("Yes") );
            ask.addButton( new JButton("No") );
            ask.setVisible(true);
            String answer = ask.getAnswer();
            ask.dispose();
            if (answer == "No")
                return;

            m_scenario.removeFaction( lFaction );
            m_scenario.buildFactionNameComboBox( m_cbFactions, true, true );
            String sel = (String)m_cbFactions.getSelectedItem();
            m_scenario.buildFactionListDisplay( m_listFactions, sel );
            
        }
        else if (bId == "Save" ) {
            saveMms();
        }
        else if (bId == "Close") {
            quit();
        }
    }

    /** mouseEntered - N/A -    Invoked when the mouse enters a component. */
    public void mouseEntered( MouseEvent me ) {
    }

    /** mouseEntered - N/A -    Invoked when the mouse exits a component. */
    public void mouseExited( MouseEvent me )  {
    }
  
    /** mousePressed - N/A - Invoked when a mouse button has been pressed on a component. */
    public void mousePressed( MouseEvent me ) {
    }
  
    /** mouseReleased - N/A - Invoked when a mouse button has been released on a component. */
    public void mouseReleased( MouseEvent me ) {
    }

    /**
     * Adds panel to frame and components for data entry to the panel
     */
    private void addPanelNorthDataEntry() {
        //Labels in a panel on the left side
        JPanel  labelsP     = new JPanel();
        //Data fields in a panel on the right side
        JPanel  dataP       = new JPanel();
        //North panel containing Labels and Data panels
        JPanel  northP      = new JPanel();
        
        //Labels
        labelsP.setLayout( new BoxLayout( labelsP, BoxLayout.Y_AXIS ) );
        labelsP.add( new Label( "MMS Version:" ) );
        labelsP.add( new Label( "Scenario Name:" ) );
        labelsP.add( new Label( "Scenario Description:" ) );
        labelsP.add( new Label( "Width:" ) );
        labelsP.add( new Label( "Height:" ) );
        labelsP.add( new Label( "Map Width:" ) );
        labelsP.add( new Label( "Map Height:" ) );
        labelsP.add( new Label( "Maps:" ) );
        labelsP.add( new Label( "Faction Filter:" ) );
        
        //Data
        dataP.setLayout( new BoxLayout( dataP, BoxLayout.Y_AXIS ) );
        m_cbVersion     = new JComboBox();
        //Currently, only 1 choice for version
        m_cbVersion.addItem("1");
        m_cbVersion.setSelectedIndex(0);
        m_tfName        = new JTextField();
        m_tfDesc        = new JTextField();
        m_tfWidth       = new JTextField();
        m_tfHeight      = new JTextField();
        m_tfMapWidth    = new JTextField();
        m_tfMapHeight   = new JTextField();
        m_tfMaps        = new JTextField();
        m_cbFactions    = new JComboBox();
        m_cbFactions.addItemListener(this);
        
        dataP.add( "m_cbVersion", m_cbVersion );
        dataP.add( "m_tfName", m_tfName );
        dataP.add( "m_tfDesc", m_tfDesc );
        dataP.add( "m_tfWidth", m_tfWidth );
        dataP.add( "m_tfHeight", m_tfHeight );
        dataP.add( "m_tfMapWidth", m_tfMapWidth );
        dataP.add( "m_tfMapHeight", m_tfMapHeight );
        dataP.add( "m_tfMaps", m_tfMaps );
        dataP.add( "m_cbFactions", m_cbFactions );
        
        //North Panel
        northP.setLayout( new BoxLayout( northP, BoxLayout.X_AXIS ) );
        northP.add( labelsP );
        northP.add( dataP );

        m_cp.add( northP, BorderLayout.NORTH );
    }

    /**
     * Adds panel to frame and components for Faction list to the panel
     */
    private void addPanelCenterFactionList() {
        //Faction list and scrollable pane
        m_listFactions = new JList();
        
        JScrollPane scrollPane = new JScrollPane(m_listFactions);
        //Center panel
        JSplitPane  centerP = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
        
//      centerP.setLayout( new BoxLayout( centerP, BoxLayout.Y_AXIS ) );
        centerP.setLeftComponent(scrollPane);

        //Debug messages
        if (m_debug) {
            m_taDebugOutput = new JTextArea();
            JScrollPane scrollPaneDebug = new JScrollPane(m_taDebugOutput);
            m_taDebugOutput.setText("Debug output area");
            centerP.setRightComponent( scrollPaneDebug );
        }
        
        m_cp.add( centerP, BorderLayout.CENTER );
    }

    /**
     * Adds panel to frame and button components to the panel
     */
    private void addPanelSouthButtons() {
        //Panel for buttons
        JPanel button1P = new JPanel();
        JPanel button2P = new JPanel();
        JPanel button3P = new JPanel();
        //South panel
        JPanel southP   = new JPanel();

        //Setup Buttons
        button1P.setLayout( new ButtonAreaLayout(true, 10) );
        button2P.setLayout( new ButtonAreaLayout(true, 10) );
        button3P.setLayout( new ButtonAreaLayout(true, 10) );

        m_bLoadScenario     = new JButton( "Load Scenario" );
        m_bAddScenario      = new JButton( "Add Scenario" );
        m_bAddFaction       = new JButton( "Add Faction" );
        m_bEditFaction      = new JButton( "Edit Faction" );
        m_bDeleteFaction    = new JButton( "Delete Faction" );
        m_bSave             = new JButton( "Save" );
        m_bClose            = new JButton( "Close" );

        m_bLoadScenario.addMouseListener( this );
        m_bAddScenario.addMouseListener( this );
        m_bAddFaction.addMouseListener( this );
        m_bEditFaction.addMouseListener( this );
        m_bDeleteFaction.addMouseListener( this );
        m_bSave.addMouseListener( this );
        m_bClose.addMouseListener( this );

        //Set default enabled states
        m_bLoadScenario.setEnabled(true);
        m_bAddScenario.setEnabled(true);
        m_bAddFaction.setEnabled(false);
        m_bEditFaction.setEnabled(false);
        m_bDeleteFaction.setEnabled(false);
        m_bSave.setEnabled(false);
        m_bClose.setEnabled(true);
        
        button1P.add( m_bLoadScenario );
        button1P.add( m_bAddScenario );
        button2P.add( m_bAddFaction );
        button2P.add( m_bEditFaction );
        button2P.add( m_bDeleteFaction );
        button3P.add( m_bSave );
        button3P.add( m_bClose );

        southP.setLayout( new BoxLayout( southP, BoxLayout.Y_AXIS ) );
        southP.add( button1P );
        southP.add( Box.createRigidArea(m_defVerSpacing) );
        southP.add( button2P );
        southP.add( Box.createRigidArea(m_defVerSpacing) );
        southP.add( button3P );
        southP.add( Box.createRigidArea(m_defVerSpacing) );

        m_cp.add( southP, BorderLayout.SOUTH );
    }
    
    /**
     * Quit application, ask to save work in progress
     */
    private void quit() {
        if (m_editsStarted) {
            int answer = askToSave();
            if (answer == M_YES) {
                answer = saveMms();
            }
            if (answer == M_CANCEL) {
                return;
            }
        }
        System.exit(0);
    }

    /**
     * Dialog to ask if current work in progress should be saved
     * @return M_YES = do save, M_NO = continue, but don't save, M_CANCEL = keep work, don't continue
     */
    private int askToSave() {
        debugMessage("Entering askToSave");
        MmsQuestion ask = new MmsQuestion(this, "Save Changes Dialog", true);
        ask.initialize();
        ask.addQuestion("Save changes?");
        ask.addButton( new JButton("Yes") );
        ask.addButton( new JButton("No") );
        ask.addButton( new JButton("Cancel") );
        ask.setVisible(true);
        String answer = ask.getAnswer();
        ask.dispose();
        if (answer == "Yes")
            return M_YES;
        else if (answer == "No")
            return M_NO;
        
        return M_CANCEL;
    }

    /**
     * Dialog to ask if current screen values should be kept
     * @return True to keep values, false to clear them 
     */
    private boolean askToKeep() {
        MmsQuestion ask = new MmsQuestion(this, "Keep Screen Values Dialog", true);
        ask.initialize();
        ask.addQuestion("Keep screen values?");
        ask.addButton( new JButton("Yes") );
        ask.addButton( new JButton("No") );
        ask.setVisible(true);
        String answer = ask.getAnswer();
        ask.dispose();
        
        return (answer == "Yes");
    }

    /**
     * Dialog to show an error
     * @param errMsg    Message to show
     */
    private void showError(String errMsg) {
        debugMessage("Entering showError");
        MmsQuestion err = new MmsQuestion(this, "Error", true);
        err.initialize();
        err.addQuestion(errMsg);
        err.addButton( new JButton("Close") );
        err.setVisible(true);
        err.dispose();
    }

    /**
     * If debug is enabled, write message to debug component
     * @param msg   Debug message to show
     */
    private void debugMessage( String msg ) {
        if (!m_debug)
            return;

        String newMsg = m_taDebugOutput.getText();
        if (newMsg.length() + msg.length() > 2048)
            newMsg = msg;
        else
            newMsg += "\n" + msg;

        m_taDebugOutput.setText( newMsg );
    }
    
    /**
     * Saves MMS info to disk
     * @return M_CANCEL if an error stops the save; else M_YES if saved ok
     */
    private int saveMms() {
        //First, update scenario with screen values
        String strVal = (String)m_cbVersion.getSelectedItem();
        m_scenario.setAttribute( "Version", strVal );
        m_scenario.setAttribute( "Name", m_tfName.getText() );
        m_scenario.setAttribute( "Desc", m_tfDesc.getText() );
        m_scenario.setAttribute( "BoardWidth", m_tfWidth.getText() );
        m_scenario.setAttribute( "BoardHeight", m_tfHeight.getText() );
        m_scenario.setAttribute( "MapWidth", m_tfMapWidth.getText() );
        m_scenario.setAttribute( "MapHeight", m_tfMapHeight.getText() );
        m_scenario.setAttribute( "Maps", m_tfMaps.getText() );
        strVal = "";
        for( int j = 1; j < m_cbFactions.getItemCount(); j++ ) {
            //Note: the value at index 0 is "All", which is why we start at 1
            if (j == 1)
                strVal = (String)m_cbFactions.getItemAt(j);
            else
                strVal += "," + (String)m_cbFactions.getItemAt(j); 
        }
        m_scenario.setAttribute( "Factions", strVal );
        
        if (m_scenario.getFileName().length() == 0) {
            FileDialog fDlg = new FileDialog(this, "Save Scenario", FileDialog.SAVE);
            //TODO - need config file for stuff like this
            //        fDlg.setDirectory( m_mmsDirectory );
            fDlg.setFilenameFilter(new FilenameFilter() {
                public boolean accept(File dir, String name) {
                    return (name != null && name.endsWith(".mms"));
                }
            });
    
            fDlg.setVisible(true);
            String fName = fDlg.getFile(); 
            if ( fName == null)
                return M_NO;
            
            fName = fDlg.getDirectory() + "/" + fName;
            m_scenario.setFileName( fName );
        }
        
        if (m_scenario.save() == false) {
            showError( m_scenario.getError() );
            return M_NO;
        }
        
        m_editsStarted = false;
        return M_YES;
    }

    /**
     * Loads MMS info from disk
     * @return M_YES if loaded ok
     */
    private int loadMms() {
        //Popup dialog to get file
        FileDialog fDlg = new FileDialog(this, "Load Scenario", FileDialog.LOAD);
        //TODO - need config file for stuff like this
        //        fDlg.setDirectory( m_mmsDirectory );
        fDlg.setFilenameFilter(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return (name != null && name.endsWith(".mms"));
            }
        });

        fDlg.setVisible(true);
        String fName = fDlg.getFile(); 
        if ( fName == null)
            return M_NO;
        
        fName = fDlg.getDirectory() + "/" + fName;
        m_scenario = new Scenario( fName, false );
        int loadOk = m_scenario.setAttributesFromFile();
        debugMessage( "Loading file status " + loadOk );
        return loadOk;
    }
    
    /**
     * Displays the MMS info
     */
    private void displayMms() {
        m_tfName.setText(m_scenario.getAttributeValue("Name"));
        m_tfDesc.setText(m_scenario.getAttributeValue("Desc"));
        m_tfWidth.setText(m_scenario.getAttributeValue("BoardWidth"));
        m_tfHeight.setText(m_scenario.getAttributeValue("BoardHeight"));
        m_tfMapWidth.setText(m_scenario.getAttributeValue("MapWidth"));
        m_tfMapHeight.setText(m_scenario.getAttributeValue("MapHeight"));
        m_tfMaps.setText(m_scenario.getAttributeValue("Maps"));

        m_scenario.buildFactionNameComboBox( m_cbFactions, true, true );
        String sel = (String)m_cbFactions.getSelectedItem();
        m_scenario.buildFactionListDisplay( m_listFactions, sel );
        
        repaint();
    }
    
    /**
     * Handles comboBox selection changes
     * @param e ItemEvent when combo box state changes
     */
    public void itemStateChanged( ItemEvent e ) {
        if (e.getStateChange() != ItemEvent.SELECTED)
            return;
        JComboBox cbEvent = (JComboBox) e.getSource();
        if (cbEvent == null)
            return;
        
        String filterName = (String) cbEvent.getSelectedItem();
        debugMessage( "Faction selected is " + filterName );

        String sel = (String)m_cbFactions.getSelectedItem();
        m_scenario.buildFactionListDisplay( m_listFactions, sel );
    }
    
    /**
     * Returns a clone of the Faction selected in the faction properties list
     * @return Null, if no valid Faction selected; otherwise a clone of the selected faction
     */
    private Faction getSelectedFaction() {
        Faction lFaction = null;
        String lFilter = (String)m_cbFactions.getSelectedItem();
        if (lFilter.compareTo("All") != 0) {
            debugMessage( "Filter string is " + lFilter );
            lFaction = m_scenario.getFactionByName( lFilter );
        } else {
            String lProp = (String)m_listFactions.getSelectedValue();
            debugMessage( "Property string is " + lProp );
            if (lProp != null) {
                lFaction = m_scenario.getFactionByProperty( lProp );
            }
        }
        
        if (lFaction == null)
            return lFaction;
        else
            return lFaction.clone();
    }
}