/*
 * MmsFactionDialog.java -  Dialog to add and edit a Faction
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.plaf.basic.BasicOptionPaneUI.*;

/**
 * Dialog to handle edits, adds, and deletes of Factions for a loaded or new Scenario
 * @author  Michael Latham
 */
public class MmsFactionDialog extends JDialog implements MouseListener, ItemListener, ListSelectionListener {

    /* Constants */
    final int M_NO          = 0;
    final int M_YES         = 1;
    final int M_CANCEL      = 2;
    final int M_MODE_ADD    = 0;
    final int M_MODE_EDIT   = 1;
    final int M_BUTTON_WIDTH    = 110;
    final int M_BUTTON_HEIGHT   = 25;

    /* Attributes */
    private Faction     m_faction;
    private int         m_mode;
    private int         m_answer;
    private Dimension   m_buttonSize;
    private boolean     m_editsStarted;
    
    /* GUI Attributes */
    //North components
    private JTextField      m_tfFactionName;
    private JTextField      m_tfFactionLocation;

    //Center components
    private JList           m_listUnits;
    private JList           m_listProperties;
    
    //South components
    private JButton         m_bOk;
    private JButton         m_bCancel;

    /* Constructors */
    /** Default constructor - not recommended */
    public MmsFactionDialog () {
        super();
        setTitle("MMS Faction Editor");
        setModal(true);
        m_faction       = new Faction();
        m_mode          = M_MODE_ADD;
        
        initialize();
    }
    
    /** 
     * Recommended constructor; note: passed Faction is cloned.
     * @param parent        Parent frame
     * @param lFaction      Faction object for E/A/D
     * @param newFaction    True if the given Faction is new (not saved to disk)
     */
    public MmsFactionDialog( Frame parent, Faction lFaction, boolean newFaction ) {
        super( parent, "MMS Faction Editor", true );
        m_faction       = lFaction.clone();
        if (newFaction)
            m_mode = M_MODE_ADD;
        else
            m_mode = M_MODE_EDIT;

        initialize();
    }
    
    /**
     * Called from constructor to initialize the attributes
     */
    private void initialize() {
        m_buttonSize    = new Dimension(M_BUTTON_WIDTH, M_BUTTON_HEIGHT);
        m_editsStarted  = false;
        m_answer        = M_CANCEL;
        buildDialog();
        displayFaction();
    }

    /** 
     * Mouse click event handler
     * @param me    MouseEvent from mouse button click
     */
    public void mouseClicked( MouseEvent me ) {
        JButton bClicked = (JButton) me.getSource();
        
        if (bClicked == null)
            return;
        
        String id = bClicked.getText();
        if (id.compareTo("OK") == 0) {
            saveFaction();
            quit();
        }
        else if (id.compareTo("Close") == 0) {
            quit();
        }
        
        //Unit E/A/D
        if (id.compareTo("Add Unit") == 0) {
            addUnit();
            m_editsStarted = true;
        }
        else if (id.compareTo("Edit Unit") == 0) {
            editUnit();
            m_editsStarted = true;
        }
        else if (id.compareTo("Delete Unit") == 0) {
            deleteUnit();
            m_editsStarted = true;
        }
        
        //Unit Options
        else if (id.compareTo("Add Option") == 0) {
            addProperty();
            m_editsStarted = true;
        }
        else if (id.compareTo("Edit Option") == 0) {
            editProperty();
            m_editsStarted = true;
        }
        else if (id.compareTo("Delete Option") == 0) {
            deleteProperty();
            m_editsStarted = true;
        }
    }

    /** mouseEntered - N/A -    Invoked when the mouse enters a component. */
    public void mouseEntered( MouseEvent me ) {
    }

    /** mouseEntered - N/A -    Invoked when the mouse exits a component. */
    public void mouseExited( MouseEvent me )  {
    }
  
    /** mousePressed - N/A - Invoked when a mouse button has been pressed on a component. */
    public void mousePressed( MouseEvent me ) {
    }
  
    /** mouseReleased - N/A - Invoked when a mouse button has been released on a component. */
    public void mouseReleased( MouseEvent me ) {
    }

    /**
     * Handles comboBox selection changes
     * @param e Event from combobox state change
     */
    public void itemStateChanged( ItemEvent e ) {
    }
    
    /**
     * Handles JList selection changes
     * @param e Event from list selection change
     */
    public void valueChanged( ListSelectionEvent e ) {
        JList jList = (JList) e.getSource();
        if (jList == null)
            return;

        displayUnitDetails();
    }

    /**
     * Build GUI display
     */
    private void buildDialog() {
        setLayout( new BorderLayout(10,10) );
        Dimension paneSize = new Dimension( 500, 500 );
        setMinimumSize( paneSize );
        setPreferredSize( paneSize );
        setSize( paneSize );
        Window win = getOwner();
        if (win != null) {
            Point loc = win.getLocation();
            setLocation( loc.x + 10, loc.y + 10 );
        }
        else {
            setLocation( 200, 100 );
        }

        addPanelNorthData();
        addPanelCenterFactionLists();
        addPanelSouthButtons();
    }

    /**
     * Layout the North panel
     */
    private void addPanelNorthData () {
        //Labels in a panel on the left side
        JPanel  labelsP     = new JPanel();
        //Data fields in a panel on the right side
        JPanel  dataP       = new JPanel();
        //North panel containing Labels and Data panels
        JPanel  northP      = new JPanel();
        
        //Labels
        labelsP.setLayout( new BoxLayout( labelsP, BoxLayout.Y_AXIS ) );
        labelsP.add( new Label( "Faction Name:" ) );
        labelsP.add( new Label( "Location:" ) );
//        labelsP.add( new Label( "Units" ) );

        //Data
        dataP.setLayout( new BoxLayout( dataP, BoxLayout.Y_AXIS ) );
        m_tfFactionName   = new JTextField();
        m_tfFactionLocation = new JTextField();
        JComboBox filterCB  = new JComboBox();
        filterCB.addItem("All");
        filterCB.addItem("BattleMech");
        filterCB.addItem("Protomech");
        filterCB.addItem("Infantry");
        filterCB.addItem("Vehicle");
        filterCB.addItem("VTOL");
        filterCB.setSelectedIndex(0);
        filterCB.addItemListener(this);
        dataP.add( "m_tfFactionName", m_tfFactionName );
        dataP.add( "m_tfFactionLocation", m_tfFactionLocation );
//        dataP.add( "filterCB", filterCB );

        //North Panel
        northP.setLayout( new BoxLayout( northP, BoxLayout.X_AXIS ) );
        northP.add( labelsP );
        northP.add( dataP );

        add( northP, BorderLayout.NORTH );
    }
    
    /**
     * Layout Center panel with Faction unit and detail lists
     */
    private void addPanelCenterFactionLists () {
        //Units Panel
        JPanel  unitP       = new JPanel();
        unitP.setLayout(  new BoxLayout( unitP, BoxLayout.X_AXIS ) );
        m_listUnits         = new JList(new DefaultListModel());
        m_listUnits.addListSelectionListener( this );
        JScrollPane unitSP  = new JScrollPane(m_listUnits);
        JPanel  unitButtonP = new JPanel();
        unitButtonP.setLayout( new BoxLayout( unitButtonP, BoxLayout.Y_AXIS ) );
        JButton addUnit     = new JButton("Add Unit");
        JButton editUnit    = new JButton("Edit Unit");
        JButton deleteUnit  = new JButton("Delete Unit");
        configButton(addUnit);
        configButton(editUnit);
        configButton(deleteUnit);
        addUnit.addMouseListener(this);
        editUnit.addMouseListener(this);
        deleteUnit.addMouseListener(this);
        unitButtonP.add( "addUnit", addUnit );
        unitButtonP.add( Box.createRigidArea( new Dimension(0,10) ) );
        unitButtonP.add( "editUnit", editUnit );
        unitButtonP.add( Box.createRigidArea( new Dimension(0,10) ) );
        unitButtonP.add( "deleteUnit", deleteUnit);
        unitP.add( unitSP );
        unitP.add( unitButtonP );
        
        //Options Panel
        JPanel  optionP       = new JPanel();
        optionP.setLayout(  new BoxLayout( optionP, BoxLayout.X_AXIS ) );
        m_listProperties = new JList();
        JScrollPane optionSP = new JScrollPane(m_listProperties);
        JPanel  optionButtonP = new JPanel();
        optionButtonP.setLayout( new BoxLayout( optionButtonP, BoxLayout.Y_AXIS ) );
        JButton addOption     = new JButton("Add Option");
        JButton editOption    = new JButton("Edit Option");
        JButton deleteOption  = new JButton("Delete Option");
        addOption.addMouseListener(this);
        editOption.addMouseListener(this);
        deleteOption.addMouseListener(this);
        configButton(addOption);
        configButton(editOption);
        configButton(deleteOption);
        optionButtonP.add( "addOption", addOption );
        optionButtonP.add( Box.createRigidArea( new Dimension(0,10) ) );
        optionButtonP.add( "editOption", editOption );
        optionButtonP.add( Box.createRigidArea( new Dimension(0,10) ) );
        optionButtonP.add( "deleteOption", deleteOption);
        optionP.add( optionSP );
        optionP.add( optionButtonP );
        
        
        //Center Panel
        JPanel centerP = new JPanel();
        centerP.setLayout(new BoxLayout( centerP, BoxLayout.Y_AXIS ));
        centerP.add( unitP );
        centerP.add( optionP );
        
        //Add center panel to our dialog
        add( centerP, BorderLayout.CENTER );
    }
    
    /**
     * Layout South panel with main function buttons
     */
    private void addPanelSouthButtons () {
        //Panel for buttons
        JPanel southP   = new JPanel();
        southP.setLayout( new BoxLayout( southP, BoxLayout.Y_AXIS ) );
        JPanel southBAL = new JPanel();
        southBAL.setLayout( new ButtonAreaLayout(true, 10) );

        JButton okB    = new JButton( "OK" );
        JButton closeB = new JButton( "Close" );
        okB.addMouseListener(this);
        closeB.addMouseListener(this);
        southBAL.add( "ok", okB );
        southBAL.add("close", closeB );

        //add South panel to our dialog
        southP.add(southBAL);
        southP.add( Box.createRigidArea(new Dimension( 0,10 )) );
        add( southP, BorderLayout.SOUTH );
    }
    /**
     * Update GUI with info from Faction object
     */
    private void displayFaction() {
        if (m_faction == null)
            return;
        
        m_tfFactionName.setText( m_faction.getName() );
        m_tfFactionLocation.setText( m_faction.getLocation() );

        displayUnits(0);
        displayUnitDetails();
        
        repaint();
    }
    
    /**
     * Display Units
     * @param selIdx    Unit index (zero based) which should be selected
     */
    private void displayUnits(int selIdx) {
        Vector<String> factionUnits = new Vector<String>();
        m_faction.addUnits( factionUnits );
        m_listUnits.setListData( factionUnits );
        m_listUnits.setSelectedIndex( selIdx );
    }
    
    /**
     * Display Unit details based on selected unit
     */
    private void displayUnitDetails() {
        int sel = m_listUnits.getSelectedIndex();
        sel++;

        Vector<String> factionProps = new Vector<String>();
        m_faction.addUnitProperties( factionProps, sel );
        m_listProperties.setListData( factionProps );
    }
    
    /**
     * One method to make all button sizes the same
     */
    private void configButton( JButton lButton ) {
        lButton.setMinimumSize( m_buttonSize );
        lButton.setPreferredSize( m_buttonSize );
        lButton.setMaximumSize( m_buttonSize );
        lButton.setSize( m_buttonSize );
    }
    
    /**
     * Closes dialog; asks to cancel quit if changes have been made and not saved
     */
    private void quit() {
        if (m_editsStarted) {
            if (askToSave() == true)
                return;
        }
        setVisible(false);
    }
    
    /**
     * Ask if user really wants to quit this dialog
     */
    private boolean askToSave() {
        MmsQuestion ask = new MmsQuestion( this, "Ask to Save Dialog" );
        ask.initialize();
        ask.addQuestion( "Changes have been made.  Do you wish to save?" );
        ask.addButton( new JButton("Yes") );
        ask.addButton( new JButton("No") );
        ask.setVisible(true);
        String answer = ask.getAnswer();
        ask.dispose();
        
        return (answer == "Yes");
    }

    /**
     * Pop-up the edit dialog and add a new Unit
     */
    private void addUnit() {
        String unitObj = "";
        boolean redisplay = false;
        MmsEditValueDialog dlg = new MmsEditValueDialog(this, "Unit Add", "Add New Unit definition.\n Do not include Unit_xxx_1=");
        dlg.addData(unitObj);
        int mechBtn = dlg.addButton("Mech...");
        int answer = -1;
        while (answer == -1) {
            dlg.setVisible(true);
            answer = dlg.getAnswer();
            if (answer == mechBtn) {
                answer = dlg.getAnswer();
                String newMech = doMechLookup();
                if (newMech.length() > 0) {
                    String oldData = dlg.getData();
                    int idx = oldData.indexOf(",");
                    if (idx > 0) {
                        oldData = newMech + oldData.substring(idx);
                    }
                    else {
                        oldData = newMech;
                    }
                    dlg.addData(oldData);
                }
                answer = -1;
            }
        }
        if (answer == dlg.M_YES) {
            String newVal = dlg.getValue();
            if (newVal.length() > 0 ) {
                redisplay = true;
                m_faction.addUnit( newVal ); 
            }
        }
        dlg.dispose();
        
        if (redisplay) {
            int unitIdx = m_listUnits.getSelectedIndex();
            displayUnits(unitIdx);
        }
    }
    
    /**
     * Delete the select Unit and all its properties.
     */
    private void deleteUnit() {
        int unitIdx = m_listUnits.getSelectedIndex() + 1;
        if (unitIdx < 1)
            return;

        boolean redisplay = false;
        MmsQuestion ques = new MmsQuestion( this, "Delete Unit Dialog" );
        ques.addButton( "Yes" );
        ques.addButton( "No" );
        ques.addQuestion( "Delete Unit and all its properties?" );
        ques.setVisible( true );
        String ans = ques.getAnswer();
        if (ans.compareTo("Yes") == 0) {
            m_faction.removeUnit( unitIdx );
            redisplay = true;
        }
        ques.dispose();

        if (redisplay) {
            displayUnits(0);
            displayUnitDetails();
        }
    }

    /**
     * Get the selected Unit and pop-up edit dialog
     */
    private void editUnit() {
        String unitObj = (String)m_listUnits.getSelectedValue();
        int unitIdx = m_listUnits.getSelectedIndex();
        boolean redisplay = false;
        if (unitObj != null) {
            MmsEditValueDialog dlg = new MmsEditValueDialog(this, "Unit Edit", "Modify Unit definition. Do not include Unit_xxx_1=");
            dlg.addData(unitObj);
            int mechBtn = dlg.addButton("Mech...");
            int answer = -1;
            while (answer == -1) {
                dlg.setVisible(true);
                answer = dlg.getAnswer();
                if (answer == mechBtn) {
                    answer = dlg.getAnswer();
                    String newMech = doMechLookup();
                    if (newMech.length() > 0) {
                        String oldData = dlg.getData();
                        int idx = oldData.indexOf(",");
                        if (idx > 0) {
                            oldData = newMech + oldData.substring(idx);
                        }
                        else {
                            oldData = newMech;
                        }
                        dlg.addData(oldData);
                    }
                    answer = -1;
                }
            }
            if (answer == dlg.M_YES) {
                redisplay = true;
                String newVal = dlg.getValue();
                m_faction.replaceUnit( unitObj, newVal ); 
            }
            dlg.dispose();
            
            if (redisplay) {
                displayUnits(unitIdx);
            }
        }
    }
    
    /**
     * Add property for selected Unit
     */
    private void addProperty() {
        int unitIdx = m_listUnits.getSelectedIndex() + 1;
        boolean redisplay = false;
        if (unitIdx > 0) {
            MmsEditValueDialog dlg = new MmsEditValueDialog(this, "Property Add", "Add Unit property. Start with PROP=VALUE");
            dlg.addData("");
            int propBtn = dlg.addButton("Prop...");
            int answer = -1;
            while (answer == -1) {
                dlg.setVisible(true);
                answer = dlg.getAnswer();
                if (answer == propBtn) {
                    answer = dlg.getAnswer();
                    String newProp = doPropLookup();
                    if (newProp.length() > 0) {
                        dlg.addData(newProp);
                    }
                    answer = -1;
                }
            }
            if (answer == dlg.M_YES) {
                redisplay = true;
                String newVal = dlg.getValue();
                m_faction.addUnitProperty( unitIdx, newVal ); 
            }
            dlg.dispose();
        }
        if (redisplay)
            displayUnitDetails();
    }
    
    /**
     * Edit selected property for selected Unit
     */
    private void editProperty() {
        int unitIdx = m_listUnits.getSelectedIndex() + 1;
        int propIdx = m_listProperties.getSelectedIndex();

        boolean redisplay = false;
        if (unitIdx > 0 && propIdx >= 0) {
            MmsEditValueDialog dlg = new MmsEditValueDialog(this, "Property Edit", "Edit Unit property. Format like PROP=VALUE");
            String oldVal = (String)m_listProperties.getSelectedValue();
            dlg.addData( oldVal );
            dlg.setVisible(true);
            int answer = dlg.getAnswer();
            if (answer == dlg.M_YES) {
                redisplay = true;
                String newVal = dlg.getValue();
                m_faction.replaceUnitProperty( unitIdx, oldVal, newVal ); 
            }
            //TODO - add button for Property lookup
            dlg.dispose();
        }
        if (redisplay)
            displayUnitDetails();
    }

    /**
     * Delete selected property from Faction Unit
     */
    private void deleteProperty() {
        int unitIdx = m_listUnits.getSelectedIndex();
        int propIdx = m_listProperties.getSelectedIndex();
        boolean redisplay = false;
        if (unitIdx >= 0 && propIdx >= 0) {
            MmsQuestion ques = new MmsQuestion( this, "Delete Unit Property" );
            ques.addButton( "Yes" );
            ques.addButton( "No" );
            ques.addQuestion( "Delete unit property?" );
            ques.setVisible( true );
            String ans = ques.getAnswer();
            if (ans.compareTo("Yes") == 0) {
                unitIdx++;
                String propVal = (String)m_listProperties.getSelectedValue();
                String prop = "Unit_" + m_faction.getName() + "_" + unitIdx + "_" + propVal;
                m_faction.removeProperty( prop );
                redisplay = true;
            }
            ques.dispose();
        }

        if (redisplay)
            displayUnitDetails();
    }
    
    /**
     * @return Faction object used during edits
     */
    public Faction getFaction() {
        return m_faction;
    }
    
    /**
     * @return Answer as set by button clicked (M_YES if Ok)
     */
    public int getAnswer() {
        return m_answer;
    }
    
    /**
     * Saves screen changes to the Faction object
     */
    private void saveFaction() {
        m_answer = M_YES;
        String oldName = m_faction.getName();
        String newName = m_tfFactionName.getText().trim();
        if (newName.compareTo(oldName) != 0) {
            m_faction.changeName(newName);
        }
        
        String newLoc = m_tfFactionLocation.getText().trim();
        m_faction.setLocation( newLoc );
        
        m_editsStarted = false; //Prevent question on quit
    }
    
    /**
     * Handle mech lookup
     * @return Value of selected mech, or "" if none
     */
    private String doMechLookup() {
        MmsMechLookupDialog dlg = new MmsMechLookupDialog( this, "Mech Lookup Dialog" );
        String newMech = "";
        dlg.setVisible(true);
        if (dlg.getAnswer() == dlg.M_YES) {
            newMech = dlg.getValue();
        }
        dlg.dispose();

        return newMech;
    }

    /**
     * Handle property lookup
     * @return Value of selected property, or "" if none
     */
    private String doPropLookup() {
        MmsPropLookupDialog dlg = new MmsPropLookupDialog( this, "Property Lookup Dialog" );
        String newProp = "";
        dlg.setVisible(true);
        if (dlg.getAnswer() == dlg.M_YES) {
            newProp = dlg.getValue();
        }
        dlg.dispose();

        return newProp;
    }
}
