/*
 * MmsMechLookupDialog.java -  Dialog class to lookup and select a Mech/Unit for a Faction
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicOptionPaneUI.*;

/**
 * Dialog to handle Unit (e.g. Mech) lookup.
 * @author  Michael Latham
 */
public class MmsMechLookupDialog extends JDialog implements MouseListener, ItemListener{
    /**
     * Constants
     */
    final int M_NO      = 0;
    final int M_YES     = 1;
    final int M_CANCEL  = 2;
    final int M_HELP    = 3;

    Container   m_cp        = null;
    JPanel      m_pData     = null;
    JPanel      m_pButtons  = null;
    Vector<String> m_listUnits = null;
    JList       m_listDisplay = null;
    JTextField  m_tfFile    = null;
    JComboBox   m_cbUnitFilter  = null;
    String      m_helpMsg   = "";
    String      m_value;
    int         m_answer;
    int         m_btnCnt;
    
    /**
     * Default constructor
     */
    public MmsMechLookupDialog () {
        super();
        initialize();
    }
     
    /**
     * Preferred constructor
     * @param frame Parent frame
     * @param title Title for new dialog
     */
    public MmsMechLookupDialog ( JFrame frame, String title ) {
        super( frame, title, true );
        initialize();
    }

    /**
     * Alternate preferred constructor
     * @param dlg   Parent dialog
     * @param title Title for new dialog
     */
    public MmsMechLookupDialog ( JDialog dlg, String title ) {
        super( dlg, title, true );
        initialize();
    }
     
     
    /**
     * Called from constructor to prepare for dialog
     */
    private void initialize() {
        m_cp = getContentPane();
        m_cp.setLayout( new BorderLayout(10,10) );
        m_pButtons      = new JPanel();
        m_pData         = new JPanel();
        m_tfFile        = new JTextField();
        m_cbUnitFilter  = new JComboBox();
        m_listUnits     = new Vector<String>();
        m_listDisplay   = new JList(new DefaultListModel());
        m_value = "";

        Dimension paneSize = new Dimension( 400, 600 );
        setMinimumSize( paneSize );
        setPreferredSize( paneSize );
        setSize( paneSize );
        Window win = getOwner();
        if (win != null) {
            Point loc = win.getLocation();
            setLocation( loc.x + 10, loc.y + 10 );
        }
        else {
            setLocation( 200, 100 );
        }

        // Data - North pane
        JPanel pLabels  = new JPanel();
        pLabels.setLayout( new BoxLayout( pLabels, BoxLayout.Y_AXIS ) );
        pLabels.add( new Label("File") );
        pLabels.add( new Label("Unit") );
        JPanel pData    = new JPanel();
        buildUnitFilter();
        pData.setLayout( new BoxLayout( pData, BoxLayout.Y_AXIS ) );
        m_tfFile.setColumns( 20 );
        m_cbUnitFilter.addItemListener(this);
        pData.add( m_tfFile );
        pData.add( m_cbUnitFilter );
        m_pData.add( pLabels );
        m_pData.add( pData );

        // List - Center pane
        JPanel centerP      = new JPanel();
        centerP.setLayout( new BoxLayout( centerP, BoxLayout.X_AXIS ) );
        JScrollPane spUnit  = new JScrollPane(m_listDisplay);
        centerP.add( Box.createRigidArea(new Dimension( 10,0 )) );
        centerP.add( spUnit );
        centerP.add( Box.createRigidArea(new Dimension( 10,0 )) );
        
        // Buttons - South pane
        m_answer = this.M_CANCEL;
        m_btnCnt = 0;
        addButton("Select");
        addButton("Cancel");
        addButton("File...");

        m_cp.add( m_pData, BorderLayout.NORTH );
        m_cp.add( centerP, BorderLayout.CENTER );
        m_cp.add( m_pButtons, BorderLayout.SOUTH );
    }
    
    /**
     * Builds the allowable filter options for the mech display list
     */
    private void buildUnitFilter() {
        m_cbUnitFilter.removeAllItems();
        m_cbUnitFilter.addItem("All");
        m_cbUnitFilter.addItem("BattleMech");
        m_cbUnitFilter.addItem("Protomech");
        m_cbUnitFilter.addItem("Infantry");
        m_cbUnitFilter.addItem("Vehicle");
        m_cbUnitFilter.addItem("VTOL");
        m_cbUnitFilter.setSelectedIndex(0);
    }
    
    /**
     * Adds a new button and its corresponding answer to the dialog
     * @param newButtonText Text to be shown on new button.  The button's number index will be returned in m_answer
     */
    public int addButton( String newButtonText ) {
        if (newButtonText == null || newButtonText.length() < 1)
            return -1;

        m_btnCnt++;
        String btnName = Integer.toString(m_btnCnt);
        JButton newButton = new JButton( newButtonText );
        newButton.setName( btnName );
        newButton.addMouseListener(this);
        m_pButtons.add( newButton );
        repaint();
        
        return m_btnCnt;
    }
    
    /**
     * Returns the answer as set by last button clicked
     * @return Button index, where caller added buttons are > 2
     */
    public int getAnswer() {
        return m_answer;
    }
    
    /**
     * Returns the text in the text field
     * @return Value of selected Mech/Unit
     */
    public String getValue() {
        return m_value;
    }
    
    /** 
     * Mouse click event handler
     * @param me    Event generated from mouse button click
     */
    public void mouseClicked( MouseEvent me ) {
        JButton bClicked = (JButton) me.getSource();
        
        if (bClicked == null)
            return;
        
        if (bClicked.getText().indexOf("File") >= 0) {
            doFileLookup();
            return;
        }

        String strAnswer = bClicked.getName();
        m_answer = Integer.parseInt(strAnswer);
        m_value  = (String)m_listDisplay.getSelectedValue();

        if (bClicked.getText().indexOf("Select") >= 0 && m_value == null) {
            showError("Please select a Unit.");
            return;
        }

        //Set m-value to be only the Unit name
        if (m_value != null && m_value.indexOf("|") > 0) {
            m_value = m_value.substring(0, m_value.indexOf("|"));
        }
        setVisible(false);
    }

    /** mouseEntered - N/A -    Invoked when the mouse enters a component. */
    public void mouseEntered( MouseEvent me ) {
    }

    /** mouseEntered - N/A -    Invoked when the mouse exits a component. */
    public void mouseExited( MouseEvent me )  {
    }
  
    /** mousePressed - N/A - Invoked when a mouse button has been pressed on a component. */
    public void mousePressed( MouseEvent me ) {
    }
  
    /** mouseReleased - N/A - Invoked when a mouse button has been released on a component. */
    public void mouseReleased( MouseEvent me ) {
    }
    
    /**
     * Handles comboBox selection changes
     * @param e Event generated from combobox state change
     */
    public void itemStateChanged( ItemEvent e ) {
        if (e.getStateChange() != ItemEvent.SELECTED)
            return;
        JComboBox cbEvent = (JComboBox) e.getSource();
        if (cbEvent == null)
            return;

        displayUnits();
    }

    /**
     * File dialog to pick mech list
     */
    private void doFileLookup() {
        //Popup dialog to get file
        FileDialog fDlg = new FileDialog(this, "Load Mech File", FileDialog.LOAD);
        //TODO - need config file for stuff like this
        //        fDlg.setDirectory( m_mmsDirectory );
        fDlg.setFilenameFilter(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                return (name != null && name.endsWith(".mms"));
            }
        });

        fDlg.setVisible(true);
        String fName = fDlg.getFile(); 
        if (fName == null)
            return;
        if (fName.indexOf(".zip") >= 0) {
            showError("Sorry, compressed files are not yet supported");
            return;
        }
        
        String filePath = fDlg.getDirectory() + fName;
        m_listUnits.removeAllElements();
        
        try {
            LineNumberReader fr = new LineNumberReader( new FileReader(filePath) );
            String newLine = fr.readLine();
            while (newLine != null) {
                m_listUnits.addElement( newLine );
                newLine = fr.readLine();
            }
            fr.close();
        }
        catch( Exception e ) {
            showError( e.getMessage() );
            return;
        }
        
        m_tfFile.setText( filePath );
        displayUnits();
    }

    /**
     * Dialog to show an error
     * @param errMsg    Message to show
     */
    private void showError(String errMsg) {
        MmsQuestion err = new MmsQuestion(this, "Error");
        err.initialize();
        err.addQuestion(errMsg);
        err.addButton( new JButton("Close") );
        err.setVisible(true);
        err.dispose();
    }

    /**
     * Displays the list of Units as determined by the filter
     */
    private void displayUnits() {
        String selFilter = (String)m_cbUnitFilter.getSelectedItem();
        if (selFilter == null || selFilter.compareTo("All") == 0) {
            m_listDisplay.setListData( m_listUnits );
        }
        else {
            Vector<String> displayVector = new Vector<String>();
            int size = m_listUnits.size();
            for (int j = 0; j < size; j++) {
                String val = m_listUnits.elementAt(j);
                if (val.indexOf( selFilter ) >= 0) {
                    displayVector.addElement( val );
                }
            }
            m_listDisplay.setListData( displayVector );
        }
    }
}
