/*
 * MmsPropLookupDialog.java -  Dialog class to lookup and select a Mech/Unit for a Faction
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.basic.BasicOptionPaneUI.*;

/**
 * Dialog to handle Property lookup.
 * @author  Michael Latham
 */
public class MmsPropLookupDialog extends JDialog implements MouseListener {
    /**
     * Constants
     */
    final int M_NO      = 0;
    final int M_YES     = 1;
    final int M_CANCEL  = 2;
    final int M_HELP    = 3;

    Container   m_cp        = null;
    JPanel      m_pData     = null;
    JPanel      m_pButtons  = null;
    Vector<String> m_listProps = null;
    JList       m_listDisplay = null;
    JTextField  m_tfProp    = null;
    String      m_helpMsg   = "";
    String      m_value;
    int         m_answer;
    int         m_btnCnt;
    
    /**
     * Default constructor
     */
    public MmsPropLookupDialog () {
        super();
        initialize();
    }
     
    /**
     * Preferred constructor
     * @param frame Parent frame
     * @param title Title for new dialog
     */
    public MmsPropLookupDialog ( JFrame frame, String title ) {
        super( frame, title, true );
        initialize();
    }

    /**
     * Alternate preferred constructor
     * @param dlg   Parent dialog
     * @param title Title for new dialog
     */
    public MmsPropLookupDialog ( JDialog dlg, String title ) {
        super( dlg, title, true );
        initialize();
    }
     
     
    /**
     * Called from constructor to prepare for dialog
     */
    private void initialize() {
        m_cp = getContentPane();
        m_cp.setLayout( new BorderLayout(10,10) );
        m_pButtons      = new JPanel();
        m_pData         = new JPanel();
        m_tfProp        = new JTextField();
        m_listProps     = new Vector<String>();
        m_listDisplay   = new JList(new DefaultListModel());
        m_value = "";

        Dimension paneSize = new Dimension( 300, 400 );
        setMinimumSize( paneSize );
        setPreferredSize( paneSize );
        setSize( paneSize );
        Window win = getOwner();
        if (win != null) {
            Point loc = win.getLocation();
            setLocation( loc.x + 10, loc.y + 10 );
        }
        else {
            setLocation( 200, 100 );
        }

        // Data - North pane
        JPanel pLabels  = new JPanel();
        pLabels.setLayout( new BoxLayout( pLabels, BoxLayout.Y_AXIS ) );
        pLabels.add( new Label("Property") );
        JPanel pData    = new JPanel();
        pData.setLayout( new BoxLayout( pData, BoxLayout.Y_AXIS ) );
        m_tfProp.setColumns( 20 );
        m_pData.add( pLabels );
        m_pData.add( pData );

        // List - Center pane
        JPanel centerP      = new JPanel();
        centerP.setLayout( new BoxLayout( centerP, BoxLayout.X_AXIS ) );
        JScrollPane spUnit  = new JScrollPane(m_listDisplay);
        centerP.add( Box.createRigidArea(new Dimension( 10,0 )) );
        centerP.add( spUnit );
        centerP.add( Box.createRigidArea(new Dimension( 10,0 )) );
        
        // Buttons - South pane
        m_answer = this.M_CANCEL;
        m_btnCnt = 0;
        addButton("Select");
        addButton("Cancel");

        m_cp.add( m_pData, BorderLayout.NORTH );
        m_cp.add( centerP, BorderLayout.CENTER );
        m_cp.add( m_pButtons, BorderLayout.SOUTH );
        
        displayProps();
    }
    
    /**
     * Adds a new button and its corresponding answer to the dialog
     * @param newButtonText Text to be shown on new button.  The button's number index will be returned in m_answer
     */
    public int addButton( String newButtonText ) {
        if (newButtonText == null || newButtonText.length() < 1)
            return -1;

        m_btnCnt++;
        String btnName = Integer.toString(m_btnCnt);
        JButton newButton = new JButton( newButtonText );
        newButton.setName( btnName );
        newButton.addMouseListener(this);
        m_pButtons.add( newButton );
        repaint();
        
        return m_btnCnt;
    }
    
    /**
     * Returns the answer as set by last button clicked
     * @return Button index, where caller added buttons are > 2
     */
    public int getAnswer() {
        return m_answer;
    }
    
    /**
     * Returns the text in the text field
     * @return Value of selected Mech/Unit
     */
    public String getValue() {
        return m_value;
    }
    
    /** 
     * Mouse click event handler
     * @param me    Event generated from mouse button click
     */
    public void mouseClicked( MouseEvent me ) {
        JButton bClicked = (JButton) me.getSource();
        
        if (bClicked == null)
            return;
        
        String strAnswer = bClicked.getName();
        m_answer = Integer.parseInt(strAnswer);
        m_value  = (String)m_listDisplay.getSelectedValue();

        if (bClicked.getText().indexOf("Select") >= 0 && m_value == null) {
            showError("Please select a Property.");
            return;
        }

        setVisible(false);
    }

    /** mouseEntered - N/A -    Invoked when the mouse enters a component. */
    public void mouseEntered( MouseEvent me ) {
    }

    /** mouseEntered - N/A -    Invoked when the mouse exits a component. */
    public void mouseExited( MouseEvent me )  {
    }
  
    /** mousePressed - N/A - Invoked when a mouse button has been pressed on a component. */
    public void mousePressed( MouseEvent me ) {
    }
  
    /** mouseReleased - N/A - Invoked when a mouse button has been released on a component. */
    public void mouseReleased( MouseEvent me ) {
    }
    
    /**
     * Dialog to show an error
     * @param errMsg    Message to show
     */
    private void showError(String errMsg) {
        MmsQuestion err = new MmsQuestion(this, "Error");
        err.initialize();
        err.addQuestion(errMsg);
        err.addButton( new JButton("Close") );
        err.setVisible(true);
        err.dispose();
    }

    /**
     * Displays the list of available properties for a unit
     */
    private void displayProps() {
        Vector<String> displayVector = new Vector<String>();
        displayVector.addElement( "Advantages" );
        displayVector.addElement( "AutoEject" );
        displayVector.addElement( "CritHit" );
        displayVector.addElement( "Damage" );
        displayVector.addElement( "DamageSpecific" );
        displayVector.addElement( "DeploymentRound" );
        displayVector.addElement( "SetAmmoTo" );
        m_listDisplay.setListData( displayVector );
    }
}
