/*
 * Scenario.java -  contains MMS data, which it validates, reads and writes
 *
 *      Copyright (C) - 2006 Michael Latham
 *      This software may be freely distributed and modified under the GNU General Public License, Version 2 or later,
 *      as published by the Free Software Foundation.
 *
 *      This software is NOT guaranteed or warrantied.  USE AT YOUR OWN RISK.  The author and/or authors cannot be held
 *      responsible for any misuse or damage.
 *
 *      NOTE: Please add revisions as file is changed!
 *
 *      REV         DATE                        DESCRIPTION
 *      -----       ----------          --------------------------------------------------
 *      00.00       2006-04-09          Initial Implementation
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.*;
import java.util.*;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JList;

/**
 * Contains the information of an MMS file.  This is the main object of manipulation for the GUIs.
 * @author  Michael Latham
 */
public class Scenario {
    
    /**
     * Constants
     */
    final int M_NO              = 0;
    final int M_YES             = 1;
    final int M_CANCEL          = 2;
    final int M_MAX_FILE_CHARS  = 16384;

    /**
     * Attributes
     */
    //File handling
    //FULLY QUALIFIED PATH TO FILE
    private String      m_fName;
    private boolean     m_isNew;
    private String      m_error;

    private String      m_version;
    private String      m_name;
    private String      m_desc;
    private String      m_boardWidth;
    private String      m_boardHeight;
    private String      m_mapWidth;
    private String      m_mapHeight;
    private String      m_maps;
    private String      m_factions;
    
    private Vector<Faction>  m_factionVector;
    
    /** Default constructor */
    public Scenario () {
        m_fName = "";
        m_isNew = true;
        initAttributes(true);
    }
    
    /** 
     * Constructor with filename (or scenario name, if new) and new indicator
     * @param fName     Filename (full path) if isNew = false; scenario name if isNew = true
     * @param isNew     False if scenario exists on disk; true if new scenario
     */
    public Scenario (String fName, boolean isNew ) {
        m_fName = fName;
        m_isNew = isNew;
        initAttributes( isNew );
    }
    
    /** 
     * Called from the constructors and initializes all the attributes
     * @param isNew True if new scenario (not on harddisk); false if file exists
     */
    private void initAttributes( boolean isNew ) {
        m_error         = "";
        
        m_version       = "MMSVersion=";
        m_name          = "Name=";
        m_desc          = "Description=";
        m_boardWidth    = "BoardWidth=";
        m_boardHeight   = "BoardHeight=";
        m_mapWidth      = "MapWidth=";
        m_mapHeight     = "MapHeight=";
        m_maps          = "Maps=";
        m_factions      = "Factions=";
        
        m_factionVector = new Vector<Faction>();
        
        //Since we won't be setting info from a file, do some more initialization
        if (isNew) {
            m_name = "Name=" + m_fName;
            m_fName = "";
        }
    }

    /** 
     * Called after object is constructed to set (or re-set) the attributes from the file
     * @return M_YES if ok; any other return is a failure
     */
    public int setAttributesFromFile() {
        if (m_fName == "")
            return M_NO;

        initAttributes(false);

        char[] chars;
        chars = new char[M_MAX_FILE_CHARS];

        try {
            FileReader fr   = new FileReader( m_fName );
            int charCt      = fr.read( chars, 0, M_MAX_FILE_CHARS );
            fr.close();
        }
        catch( IOException e ) {
            MmsQuestion err = new MmsQuestion(null, "File error", true);
            err.initialize();
            err.addQuestion( e.getMessage() );
            err.addButton( new JButton("Close") );
            err.setVisible(true);
            err.dispose();
            return M_NO;
        }
        
        String str  = new String(chars);
        str         = str.trim();

        int i           = 0;
        int idx0        = 0;
        int idx1        = 0;
        int idx2        = 0;
        String eol      = System.getProperty("line.separator");
        String tmpStr   = "";
        String mmsParam = "";
        while (idx0 < str.length() && idx1 >= 0) {
            if (idx0 >= str.length())
                break;
            idx1 = str.indexOf( eol, idx0 );
            if (idx1 < 0)
                idx1 = str.length();
            //Skip comments
            if (str.substring(idx0,idx1).indexOf("#", 0) >= 0) {
                idx0 = idx1 + eol.length();
                continue;
            }
            i++;
            switch(i) {
                case 1:
                    m_version = str.substring(idx0,idx1);
                    idx0 = idx1 + eol.length();
                    break;
                case 2:
                    m_name = str.substring(idx0,idx1);
                    idx0 = idx1 + eol.length();
                    break;
                case 3:
                    m_desc = str.substring(idx0,idx1);
                    idx0 = idx1 + eol.length();
                    break;
                default:
                    //Since there are optional settings, now we have to parse the string
                    tmpStr = str.substring(idx0,idx1);
                    idx2 = tmpStr.indexOf( "=", 0 );
                    if (idx2 > 0) {
                        mmsParam = tmpStr.substring(0,idx2);
                        setAttributeFromParam( mmsParam.trim(), tmpStr.trim() );
                    }
                    idx0 = idx1 + eol.length();
                    break;
            } //End switch(i)
        } //End for(i=1...)

        return M_YES;
    }
    
    /**
     * Given the parameter name and the string for the parameter, set the correct attribute
     * @param mmsParam  Parameter that the line is for (e.g. BoardWidth)
     * @param mmsLine   The line for the attribute (e.g. BoardWidth=1)
     */
    public void setAttributeFromParam( String mmsParam, String mmsLine ) {
        if (mmsParam == "")
            return;
        
        if (mmsParam.compareTo("BoardWidth") == 0)
            m_boardWidth = mmsLine;
        else if (mmsParam.compareTo("BoardHeight") == 0)
            m_boardHeight = mmsLine;
        else if (mmsParam.compareTo("MapWidth") == 0)
            m_mapWidth = mmsLine;
        else if (mmsParam.compareTo("MapHeight") == 0)
            m_mapHeight = mmsLine;
        else if (mmsParam.compareTo("Maps") == 0)
            m_maps = mmsLine;
        else if (mmsParam.compareTo("Factions") == 0) {
            m_factions = mmsLine;
            buildFactionVector( mmsLine );
        }
        else
            setFactionAttribute( mmsParam, mmsLine );
    }
    /**
     * Build the faction vector of Faction objects given the list of factions
     * @param factionList   List of factions (e.g. Factions=Rangers,Scouts)
     */
    public void buildFactionVector( String factionList ) {
        m_factionVector.clear();
        
        int idx0 = factionList.indexOf( "=", 0 );
        if (idx0 > 0) {
            String factions = factionList.substring(idx0+1);
            factions = factions.trim();
            if (factions.length() > 0) {
                int idx1 = 0;
                int idx2 = 0;
                String faction;
                while (idx1 < factions.length() && idx2 >= 0) {
                    if (idx1 >= factions.length())
                        break;
                    idx2 = factions.indexOf( ",", idx1 );
                    if (idx2 < 0) 
                        idx2 = factions.length();

                    faction = factions.substring(idx1,idx2);
                    faction = faction.trim();
                    idx1 = idx2 + 1;
                    
                    Faction factionObj = new Faction(faction);
                    m_factionVector.addElement( factionObj );
                }
            }
        }
    }

    /**
     * Given the parameter name and the string for the parameter, set the correct Faction attribute
     * @param mmsParam  Parameter that the line is for (e.g. Unit_Rangers_1)
     * @param mmsLine   The line for the attribute (e.g. Unit_Rangers_1=Stone Rhino SERO,Capt Black,3,3)
     */
    public void setFactionAttribute( String mmsParam, String mmsLine ) {
        int size = m_factionVector.size();
        Faction fObj;
        String facName;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            facName = fObj.getName();
            if (mmsParam.indexOf(facName) >= 0) {
                fObj.addFactionProperty( mmsLine );
                break;
            }
        }
    }

    /**
     * Returns the desired attribute
     * @param attrib     Attribute parameter name
     * @return Attribute string
     */
    public String getAttribute( String attrib ) {
        if (attrib.compareTo( "Version" ) == 0 || attrib.compareTo( "MMSVersion" ) == 0)
            return m_version;
        if (attrib.compareTo( "Name" ) == 0)
            return m_name;
        if (attrib.compareTo( "Desc" ) == 0 || attrib.compareTo( "Description" ) == 0)
            return m_desc;
        if (attrib.compareTo( "BoardWidth" ) == 0)
            return m_boardWidth;
        if (attrib.compareTo( "BoardHeight" ) == 0)
            return m_boardHeight;
        if (attrib.compareTo( "MapWidth" ) == 0)
            return m_mapWidth;
        if (attrib.compareTo( "MapHeight" ) == 0)
            return m_mapHeight;
        if (attrib.compareTo( "Maps" ) == 0)
            return m_maps;
        if (attrib.compareTo( "Factions" ) == 0)
            return m_factions;
        
        return "";
    }

    /**
     * Set the desired attribute
     * @param attrib    Attribute parameter name (e.g. Version)
     * @param val       Attribute value (e.g. 1)
     */
    public void setAttribute( String attrib, String val ) {
        if (attrib.compareTo( "Version" ) == 0 || attrib.compareTo( "MMSVersion" ) == 0)
            m_version = "MMSVersion=" + val;
        if (attrib.compareTo( "Name" ) == 0)
            m_name = "Name=" + val;
        if (attrib.compareTo( "Desc" ) == 0 || attrib.compareTo( "Description" ) == 0)
            m_desc = "Description=" + val;
        if (attrib.compareTo( "BoardWidth" ) == 0)
            m_boardWidth = "BoardWidth=" + val;
        if (attrib.compareTo( "BoardHeight" ) == 0)
            m_boardHeight = "BoardHeight=" + val;
        if (attrib.compareTo( "MapWidth" ) == 0)
            m_mapWidth = "MapWidth=" + val;
        if (attrib.compareTo( "MapHeight" ) == 0)
            m_mapHeight = "MapHeight=" + val;
        if (attrib.compareTo( "Maps" ) == 0)
            m_maps = "Maps=" + val;
        if (attrib.compareTo( "Factions" ) == 0)
            m_factions = "Factions=" + val;
        
    }

    /**
     * returns the desired attribute value (i.e. after the "=")
     * @param attrib    Attribute parameter name
     * @return Attribute value string
     */
    public String getAttributeValue( String attrib ) {
        String retVal = "";
        int idx = 0;
        if (attrib == "Version" || attrib == "MMSVersion") {
            idx = m_version.indexOf("=") + 1;
            if (idx < m_version.length())
                retVal = m_version.substring(idx);
        }
        if (attrib == "Name") {
            idx = m_name.indexOf("=") + 1;
            if (idx < m_name.length())
                retVal = m_name.substring(idx);
        }
        if (attrib == "Desc" || attrib == "Description") {
            idx = m_desc.indexOf("=") + 1;
            if (idx < m_desc.length())
                retVal = m_desc.substring(idx);
        }
        if (attrib == "BoardWidth") {
            idx = m_boardWidth.indexOf("=") + 1;
            if (idx < m_boardWidth.length())
                retVal = m_boardWidth.substring(idx);
        }
        if (attrib == "BoardHeight") {
            idx = m_boardHeight.indexOf("=") + 1;
            if (idx < m_boardHeight.length())
                retVal = m_boardHeight.substring(idx);
        }
        if (attrib == "MapWidth") {
            idx = m_mapWidth.indexOf("=") + 1;
            if (idx < m_mapWidth.length())
                retVal = m_mapWidth.substring(idx);
        }
        if (attrib == "MapHeight") {
            idx = m_mapHeight.indexOf("=") + 1;
            if (idx < m_mapHeight.length())
                retVal = m_mapHeight.substring(idx);
        }
        if (attrib == "Maps") {
            idx = m_maps.indexOf("=") + 1;
            if (idx < m_maps.length())
                retVal = m_maps.substring(idx);
        }
        if (attrib == "Factions") {
            idx = m_factions.indexOf("=") + 1;
            if (idx < m_factions.length())
                retVal = m_factions.substring(idx);
        }
        
        return retVal;
    }
    
    /**
     * Builds a list of Faction information in the given list
     * @param factionList   List object to hold Faction information
     * @param factionFilter Name by which to filter; blank or "All" will add all
     */
    public void buildFactionListDisplay( JList factionList, String factionFilter ) {
        Vector<String> factionParams = new Vector<String>();

        int size = m_factionVector.size();
        Faction fObj;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            if (fObj != null) {
                if (factionFilter == "" || factionFilter == "All" || factionFilter.compareTo(fObj.getName()) == 0)
                    fObj.addAllProperties( factionParams );
            }
        }
         
        factionList.setListData(factionParams);
     }
    
    /**
     * Builds a combo box of Faction names; sets selected to first item
     * @param cbNames       Combo to be built
     * @param addAllText    True will add text "All" to beginning of list
     * @param clearCombo    True will clear list, first
     */
    public void buildFactionNameComboBox( JComboBox cbNames, boolean addAllText, boolean clearCombo ) {
        int newSel = 0;
        if (cbNames == null)
            return;
        //Even if we reset the combo, save any selection so we can set selected on the new list
        String oldSel = (String)cbNames.getSelectedItem();
        if (clearCombo == true)
            cbNames.removeAllItems();
        if (addAllText)
            cbNames.addItem("All");

        int size = m_factionVector.size();
        Faction fObj;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            if (fObj != null) {
                String newName = fObj.getName();
                cbNames.addItem( newName );
                if (oldSel != null && newName.compareTo(oldSel) == 0)
                    newSel = j + 1;
            }
        }
        
        cbNames.setSelectedIndex(newSel);
    }
    
    /**
     * Returns the Faction given the exact name
     * @return Faction object from the given name
     */
    public Faction getFactionByName( String lName ) {
        Faction lFaction = null;
        if (lName != null) {
            int size = m_factionVector.size();
            Faction fObj;
            for (int j = 0; j < size; j++) {
                fObj = m_factionVector.get(j);
                if (lName.compareTo(fObj.getName()) == 0) {
                    lFaction = fObj;
                    break;
                }
            }
        }
        
        return lFaction;
    }
    /**
     * Returns the Faction given a full property string
     * @return Faction object that contains the given property
     */
    public Faction getFactionByProperty( String lProp ) {
        Faction lFaction = null;
        if (lProp != null) {
            int size = m_factionVector.size();
            Faction fObj;
            for (int j = 0; j < size; j++) {
                fObj = m_factionVector.get(j);
                if (lProp.indexOf(fObj.getName()) >= 0) {
                    //May want to make sure the index is < index of the equal sign
                    //to make sure we have a Faction and not a pilot or unit
                    lFaction = fObj;
                    break;
                }
            }
        }
        
        return lFaction;
    }
    
    /**
     * Replace an existing faction with a new faction
     * @param oldFaction    Old Faction to be replaced
     * @param newFaction    New Faction to take the old one's place
     */
    public void replaceFaction( Faction oldFaction, Faction newFaction ) {
        int size = m_factionVector.size();
        String lName = oldFaction.getName();
        Faction fObj;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            if (lName.compareTo(fObj.getName()) == 0) {
                //Got it, and j is the index
                m_factionVector.removeElementAt( j );
                m_factionVector.add( j, newFaction );
                break;
            }
        }
    }
    
    /**
     * Add the given faction to the faction vector
     * @param newFaction    New Faction object to add to the Faction list
     */
    public void addFaction ( Faction newFaction ) {
        if (newFaction == null)
            return;
        
        m_factionVector.add( newFaction );
    }

    /**
     * Removes the given faction from the faction vector
     * @param delFaction    Faction object to be deleted from Faction list
     */
    public void removeFaction( Faction delFaction ) {
        if (delFaction == null)
            return;
        int size = m_factionVector.size();
        String lName = delFaction.getName();
        Faction fObj;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            if (lName.compareTo(fObj.getName()) == 0) {
                //Got it, and j is the index
                m_factionVector.removeElementAt( j );
                break;
            }
        }
    }
    
    /**
     * @return Current filename (full path) for the Scenario
     */
    public String getFileName() {
        return m_fName;
    }

    /**
     * Set Scenario filename
     * @param fName Filename (full path) for the Scenario
     */
    public void setFileName( String fName ) {
        m_fName = fName;
    }
    
    /**
     * @return Last error
     */
    public String getError() {
        return m_error;
    }

    /**
     * Saves the scenario to disk.  Call getError() if save returns false.
     * @return True if successful save; false otherwise
     */
    public boolean save() {
        if (m_fName.length() == 0) {
            m_error = "No file given.";
            return false;
        }
        // Prepare output string
        String outStr = "";
        String lineSep = System.getProperty("line.separator");
        outStr += m_version + lineSep;
        outStr += m_name + lineSep;
        outStr += m_desc + lineSep;
        outStr += m_boardWidth + lineSep;
        outStr += m_boardHeight + lineSep;
        if (getAttributeValue("MapWidth").length() > 0)
            outStr += m_mapWidth + lineSep;
        if (getAttributeValue("MapHeight").length() > 0)
            outStr += m_mapHeight + lineSep;
        outStr += m_maps + lineSep;
        outStr += m_factions + lineSep;

        // Add each Faction to the scenario
        int size = m_factionVector.size();
        Faction fObj;
        for (int j = 0; j < size; j++) {
            fObj = m_factionVector.get(j);
            if (fObj != null)
                outStr += fObj.getFactionForScenarioFile();
        }

        // Write output to file; we will wholly replace an existing file
        try {
            FileWriter fw = new FileWriter( m_fName, false );
            fw.write( outStr, 0, outStr.length() );
            fw.close();
        }
        catch (Exception e) {
            m_error = e.getMessage();
            return false;
        }
        
        return true;
    }
    
    /**
     * Set Scenario as a new scenario, while maintaining most attributes
     */
    public void setAsNew() {
        m_fName = "";
        m_name  = "New Scenario";
        m_isNew = true;
        m_error = "";
    }

}
